package io.gitlab.jfronny.respackopts;

import com.mojang.brigadier.Command;
import io.gitlab.jfronny.respackopts.integration.SaveHook;
import io.gitlab.jfronny.respackopts.util.MetaCache;
import net.fabricmc.fabric.api.client.command.v2.ClientCommandRegistrationCallback;
import net.fabricmc.fabric.api.client.command.v2.FabricClientCommandSource;
import net.fabricmc.loader.api.*;
import net.minecraft.text.Text;

import java.io.BufferedWriter;
import java.nio.file.*;
import java.util.concurrent.CompletableFuture;

import static net.fabricmc.fabric.api.client.command.v2.ClientCommandManager.literal;

public class RpoClientCommand {
    private static final Version VERSION = FabricLoader.getInstance().getModContainer(Respackopts.ID).get().getMetadata().getVersion();
    public static void register() {
        ClientCommandRegistrationCallback.EVENT.register((dispatcher, registryAccess) -> {
            Command<FabricClientCommandSource> getVersion = ctx -> {
                ctx.getSource().sendFeedback(Text.translatable("respackopts.versionText", VERSION, Respackopts.META_VERSION));
                return 1;
            };
            Command<FabricClientCommandSource> dumpConfig = ctx -> {
                MetaCache.forEach((id, branch) -> ctx.getSource().sendFeedback(dump(branch.toString(), id + ".txt")));
                return 1;
            };
            Command<FabricClientCommandSource> dumpGlsl = ctx -> {
                ctx.getSource().sendFeedback(dump(RespackoptsClient.getShaderImportSource(), "frex.glsl"));
                return 1;
            };
            Command<FabricClientCommandSource> reload = ctx -> {
                MetaCache.clear();
                CompletableFuture.allOf(RespackoptsClient.forceReloadResources(), RespackoptsClient.reloadIntegratedServerData())
                        .thenRun(() -> {
                            ctx.getSource().sendFeedback(Text.translatable("respackopts.reloadSucceeded"));
                        }).exceptionally(e -> {
                            Respackopts.LOGGER.error("Could not reload resources", e);
                            ctx.getSource().sendError(Text.translatable("respackopts.reloadFailed"));
                            return null;
                        });
                return 1;
            };
            dispatcher.register(literal("rpoc").executes(getVersion)
                    .then(literal("dump").executes(dumpConfig)
                            .then(literal("config").executes(dumpConfig))
                            .then(literal("glsl").executes(dumpGlsl)))
                    .then(literal("version").executes(getVersion))
                    .then(literal("reload").executes(reload)));
        });
    }

    private static final Path dumpPath = FabricLoader.getInstance().getGameDir().resolve("respackopts");
    private static Text dump(String text, String fileName) {
        try {
            if (!Files.exists(dumpPath)) Files.createDirectories(dumpPath);
            Path filePath = dumpPath.resolve(fileName);
            try (BufferedWriter bw = Files.newBufferedWriter(filePath, StandardOpenOption.CREATE)) {
                bw.write(text);
            }
            return Text.translatable("respackopts.dumpSucceeded", filePath.toAbsolutePath());
        }
        catch (Throwable e) {
            Respackopts.LOGGER.error("Could not dump resource", e);
            return Text.translatable("respackopts.dumpFailed");
        }
    }
}
