package io.gitlab.jfronny.respackopts.integration;

import io.gitlab.jfronny.respackopts.Respackopts;
import io.gitlab.jfronny.respackopts.RespackoptsConfig;
import net.fabricmc.loader.api.FabricLoader;

import java.io.IOException;
import java.nio.file.*;
import java.nio.file.attribute.BasicFileAttributes;

public class DashLoaderCompat {
    public static void requestForceReload() {
        if (!FabricLoader.getInstance().isModLoaded("dashloader"))
            return;
        if (!RespackoptsConfig.dashloaderCompat)
            return;
        try {
            if (RespackoptsConfig.debugLogs)
                Respackopts.LOGGER.info("Removing DashCache to force dashloader to reload");
            Class<?> loaderClass = Class.forName("dev.quantumfusion.dashloader.def.DashLoader");
            Path loaderPath = (Path) loaderClass.getDeclaredField("DASH_CACHE_FOLDER").get(null);
            Files.walkFileTree(loaderPath, new SimpleFileVisitor<>() {
                @Override
                public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException {
                    Files.delete(file);
                    return FileVisitResult.CONTINUE;
                }

                @Override
                public FileVisitResult postVisitDirectory(Path dir, IOException exc) throws IOException {
                    Files.delete(dir);
                    return FileVisitResult.CONTINUE;
                }
            });
        } catch (Throwable e) {
            Respackopts.LOGGER.error("Failed to remove dashloader data, try disabling dashloaderCompat if this keeps happening", e);
        }
    }
}
