package io.gitlab.jfronny.respackopts.filters.util;

import io.gitlab.jfronny.libjf.ResourcePath;
import io.gitlab.jfronny.respackopts.Respackopts;
import io.gitlab.jfronny.respackopts.gson.*;
import io.gitlab.jfronny.respackopts.model.DirRpo;
import io.gitlab.jfronny.respackopts.model.FileRpo;
import io.gitlab.jfronny.respackopts.model.cache.*;
import io.gitlab.jfronny.respackopts.util.MetaCache;
import net.minecraft.resource.ResourceNotFoundException;
import net.minecraft.resource.ResourcePack;

import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Map;

public class FileRpoSearchProvider {
    public static boolean isRpo(String fileName) {
        return fileName.endsWith(Respackopts.FILE_EXTENSION);
    }

    public static <T> T modifyWithRpo(String fileName, ResourcePack pack, ModifiedGenerator<T> getModified, T defaultValue) {
        if (FileRpoSearchProvider.isRpo(fileName))
            return defaultValue;
        CachedPackState state = MetaCache.getState(MetaCache.getKeyByPack(pack));
        Map<String, FileRpo> frpReg = state.cachedFileRPOs();
        String rpPathName = fileName + Respackopts.FILE_EXTENSION;
        if (frpReg.containsKey(rpPathName))
            return getModified.getModified(frpReg.get(rpPathName));
        ResourcePath rpoPath = null;
        if (fileName.contains("/")) {
            try {
                rpoPath = new ResourcePath(rpPathName);
                if (!pack.contains(rpoPath.getType(), rpoPath.getId()))
                    return defaultValue;
            } catch (Throwable e) {
                Respackopts.LOGGER.error("Could not check file filter status", e);
                return defaultValue;
            }
        }
        try (InputStream stream = rpoPath == null ? pack.openRoot(rpPathName) : pack.open(rpoPath.getType(), rpoPath.getId());
             Reader w = stream == null ? null : new InputStreamReader(stream)) {
            if (w == null) throw new FileNotFoundException("Could not find file: " + fileName);
            FileRpo frp = AttachmentHolder.deserialize(state.metadata().version, w, FileRpo.class);
            frp.path = rpPathName;
            frpReg.put(rpPathName, frp);
            return getModified.getModified(frp);
        }
        catch (Exception e) {
            if (!(e instanceof ResourceNotFoundException)) Respackopts.LOGGER.error("Could not generate replacement for " + (rpoPath == null ? fileName : rpoPath.getName()) + " in " + pack.getName(), e);
            return defaultValue;
        }
    }

    public interface ModifiedGenerator<T> {
        T getModified(FileRpo rpo);
    }
}
