package io.gitlab.jfronny.respackopts.gson;

import io.gitlab.jfronny.commons.throwable.*;
import io.gitlab.jfronny.respackopts.*;

import java.io.*;

public class AttachmentHolder {
    private static final ThreadLocal<Attachment> attachments = new ThreadLocal<>();

    public static <TEx extends Throwable> void attach(int version, ThrowingRunnable<TEx> runnable) throws TEx {
        try {
            attachments.set(new Attachment(version));
            runnable.run();
        } finally {
            attachments.remove();
        }
    }

    public static <TOut, TEx extends Throwable> TOut attach(int version, ThrowingSupplier<TOut, TEx> runnable) throws TEx {
        try {
            attachments.set(new Attachment(version));
            return runnable.get();
        } finally {
            attachments.remove();
        }
    }

    public static <T> T deserialize(int version, Reader r, Class<T> klazz) {
        return attach(version, () -> Respackopts.GSON.fromJson(r, klazz));
    }

    public static int getAttachedVersion() {
        return attachments.get().version;
    }

    private record Attachment(int version) {
    }
}
