package io.gitlab.jfronny.respackopts.model;

import io.gitlab.jfronny.commons.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.compiler.expr.*;
import io.gitlab.jfronny.muscript.error.*;

public record Condition(String source, BoolExpr expr) {
    public boolean get(Dynamic<?> dataRoot) throws ConditionException {
        try {
            return expr.get(dataRoot);
        } catch (LocationalException e) {
            throw new ConditionException(e.asPrintable(source), e);
        } catch (RuntimeException e) {
            throw new ConditionException(LocationalError.create(source, 0, e.getMessage()), e);
        }
    }

    public static class ConditionException extends Exception {
        public final LocationalError error;

        public ConditionException(LocationalError error, Throwable cause) {
            super(error.toString(), cause);
            this.error = error;
        }
    }
}
